# Copyright @yucwang 2021

import Imath
import json
import numpy as np
import os
import sys
import OpenEXR

from exr_utils import read_exr_file

def compare_two_files(image_path, ref_image_path, x_min = -1, x_max = -1, y_min = -1, y_max = -1):
    (r, g, b) = read_exr_file(image_path)
    (rr, rg, rb) = read_exr_file(ref_image_path)

    if x_min < 0:
        x_min = 0
    if x_max < 0:
        x_max = r.shape[1]
    if y_min < 0:
        y_min = 0
    if y_max < 0:
        y_max = r.shape[0]

    r = r[y_min:y_max, x_min:x_max]
    g = g[y_min:y_max, x_min:x_max]
    b = b[y_min:y_max, x_min:x_max]
    rr = rr[y_min:y_max, x_min:x_max]
    rg = rg[y_min:y_max, x_min:x_max]
    rb = rb[y_min:y_max, x_min:x_max]

    re = (r - rr)
    ge = (g - rg)
    be = (b - rb)
    relMAE = (np.mean(np.absolute(re) / (rr + 1e-2)) + np.mean(np.absolute(ge) / (rg + 1e-2)) + np.mean(np.absolute(be) / (rb + 1e-2))) / 3.0
    relMSE = (np.mean((re * re) / (rr * rr + 1e-4)) + np.mean((ge * ge) / (rg * rg + 1e-4)) + np.mean((be * be) / (rb * rb + 1e-4))) / 3.0

    return { "souce": os.path.basename(image_path), "ref": os.path.basename(ref_image_path), "relMAE": relMAE, "relMSE": relMSE }

if __name__ == "__main__":
    source_images = []
    ref_image = ""
    argc = len(sys.argv)
    if argc <= 2:
        print("python3 comarison_exr.py [source images] [ref images].")

    state = "file_path"
    (x_min, x_max) = (-1, -1)
    (y_min, y_max) = (-1, -1)
    for i, arg in enumerate(sys.argv):
        if i > 0 and i < argc - 1:
            if state == "x_min":
                x_min = int(arg)
                state = "x_max"
            elif state == "x_max":
                x_max = int(arg)
                state = "file_path"
            elif state == "y_min":
                y_min = int(arg)
                state = "y_max"
            elif state == "y_max":
                y_max = int(arg)
                state = "file_path"
            elif state == "file_path":
                if arg == "--x":
                    state = "x_min"
                elif arg == "--y":
                    state = "y_min"
                else:
                    source_images.append(arg)
        elif i == argc - 1:
            ref_image = arg

    for source_image in source_images:
        result = compare_two_files(source_image, ref_image, x_min, x_max, y_min, y_max)
        print(json.dumps(result, indent=4))
