# Copyright @yucwang 2021

import array
import Imath
import numpy as np
import OpenEXR

def write_exr_file(file_name, np_r, np_g, np_b):
    (w, h) = np_r.shape
    exr_file = OpenEXR.OutputFile(file_name, OpenEXR.Header(h, w))

    r = np_r.astype(np.float32).tobytes()
    g = np_g.astype(np.float32).tobytes()
    b = np_b.astype(np.float32).tobytes()
    exr_file.writePixels({'R': r, 'G': g, 'B': b})
    exr_file.close()

def read_exr_file(file_path):
    exr_file = OpenEXR.InputFile(file_path)
    exr_header = exr_file.header()

    dataWindow = exr_header["dataWindow"]
    (w, h) = (dataWindow.max.x - dataWindow.min.x + 1, dataWindow.max.y - dataWindow.min.y + 1)

    EXR_FLOAT = Imath.PixelType(Imath.PixelType.FLOAT)

    np_r = np.fromstring(exr_file.channel('R', EXR_FLOAT), dtype=np.float32).reshape(h, w)
    np_g = np.fromstring(exr_file.channel('G', EXR_FLOAT), dtype=np.float32).reshape(h, w)
    np_b = np.fromstring(exr_file.channel('B', EXR_FLOAT), dtype=np.float32).reshape(h, w)

    exr_file.close()
    return np_r, np_g, np_b
