# Copyright @yucwang 2021

import numpy as np
import os
import sys

from exr_utils import read_exr_file, write_exr_file

def crop(image_path, x_min = -1, x_max = -1, y_min = -1, y_max = -1):
    (r, g, b) = read_exr_file(image_path)
    basename = os.path.basename(image_path)
    filename = "{}_{}_{}_{}_{}.exr".format(basename, x_min, x_max, y_min, y_max)

    r = r[y_min:y_max, x_min:x_max]
    g = g[y_min:y_max, x_min:x_max]
    b = b[y_min:y_max, x_min:x_max]

    write_exr_file(filename, r, g, b)

def highlight(image_path, x_min = -1, x_max = -1, y_min = -1, y_max = -1, color = [1.0, 0.0, 0.0], bold=3):
    (r, g, b) = read_exr_file(image_path)

    x_min = max(x_min, 0)
    y_min = max(y_min, 0)
    x_max = min(x_max, r.shape[1])
    y_max = min(y_max, r.shape[0])
    if x_max < 0:
        x_max = r.shape[1]
    if y_max < 0:
        y_max = r.shape[0]

    # Horizontal Line
    # upper
    xx = min(x_min + bold, r.shape[1])
    r[y_min:y_max, x_min:xx] = color[0]
    g[y_min:y_max, x_min:xx] = color[1]
    b[y_min:y_max, x_min:xx] = color[2]

    # lower
    xx = max(x_max - bold, 0)
    r[y_min:y_max, xx:x_max] = color[0]
    g[y_min:y_max, xx:x_max] = color[1]
    b[y_min:y_max, xx:x_max] = color[2]

    # Vertical Line
    # left
    yy = min(y_min + bold, r.shape[0])
    r[y_min:yy, x_min:x_max] = color[0]
    g[y_min:yy, x_min:x_max] = color[1]
    b[y_min:yy, x_min:x_max] = color[2]

    # right
    yy = max(y_max - bold, 0)
    r[yy:y_max, x_min:x_max] = color[0]
    g[yy:y_max, x_min:x_max] = color[1]
    b[yy:y_max, x_min:x_max] = color[2]

    filename = "{}_hightlighted.exr".format(os.path.basename(image_path))
    write_exr_file(filename, r, g, b)

def parse_crop(argc, argv):
    state = "file_path"
    (x_min, x_max) = (-1, -1)
    (y_min, y_max) = (-1, -1)
    for i, arg in enumerate(argv):
        if i > 1:
            if state == "x_min":
                x_min = int(arg)
                state = "x_max"
            elif state == "x_max":
                x_max = int(arg)
                state = "file_path"
            elif state == "y_min":
                y_min = int(arg)
                state = "y_max"
            elif state == "y_max":
                y_max = int(arg)
                state = "file_path"
            elif state == "file_path":
                if arg == "--x":
                    state = "x_min"
                elif arg == "--y":
                    state = "y_min"
                else:
                    file_path = arg
    return (file_path, x_min, x_max, y_min, y_max)

def parse_highlight(argc, argv):
    state = "file_path"
    (x_min, x_max) = (-1, -1)
    (y_min, y_max) = (-1, -1)
    bold = 5
    color = [1.0, 0.0, 0.0]
    for i, arg in enumerate(argv):
        if i > 1:
            if state == "x_min":
                x_min = int(arg)
                state = "x_max"
            elif state == "x_max":
                x_max = int(arg)
                state = "file_path"
            elif state == "y_min":
                y_min = int(arg)
                state = "y_max"
            elif state == "y_max":
                y_max = int(arg)
                state = "file_path"
            elif state == "bold":
                bold = int(arg)
                state = "file_path"
            elif state == "red":
                color[0] = float(arg)
                state = "green"
            elif state == "green":
                color[1] = float(arg)
                state = "blue"
            elif state == "blue":
                color[2] = float(arg)
                state = "file_path"
            elif state == "file_path":
                if arg == "--x":
                    state = "x_min"
                elif arg == "--y":
                    state = "y_min"
                elif arg == "--bold":
                    state = "bold"
                elif arg == "--color":
                    state = "red"
                else:
                    file_path = arg
    return (file_path, x_min, x_max, y_min, y_max, color, bold)

if __name__ == "__main__":
    argc = len(sys.argv)
    argv = sys.argv

    if argv[1] == "highlight":
        (file_path, x_min, x_max, y_min, y_max, color, bold) = parse_highlight(argc, argv)
        highlight(file_path, x_min, x_max, y_min, y_max, color, bold)
    elif argv[1] == "crop":
        (file_path, x_min, x_max, y_min, y_max) = parse_crop(argc, argv)
        crop(file_path, x_min, x_max, y_min, y_max)
    else:
        print("Command not found, should be [highlight] or [crop].")

# crop("/home/yucwang/Documents/development/pbrt-mirror/build/bathroom_varl_2.exr", 570, 670, 120, 220)
# crop("/home/yucwang/Documents/development/bisml/build/bathroom_bomis_2.exr", 570, 670, 120, 220)
# crop("/home/yucwang/Documents/development/research/many-light_rendering/gt/bathroom_2.exr", 570, 670, 120, 220)
